function SearchReplacePlugin() {
    const { useState, useEffect } = wp.element;
    const { __, sprintf } = wp.i18n;
    const { select, dispatch } = wp.data;

    const [searchText, setSearchText] = useState('');
    const [replaceText, setReplaceText] = useState('');
    const [useRegex, setUseRegex] = useState(false);
    const [caseSensitive, setCaseSensitive] = useState(false);
    const [matches, setMatches] = useState([]);
    const [currentMatch, setCurrentMatch] = useState(0);


    useEffect(() => {
        return () => {
            cleanupHighlights();
        };
    }, []);

    const cleanupHighlights = () => {
        try {
            const blocks = select('core/block-editor').getBlocks();
            blocks.forEach(block => {
                if (!block.attributes?.content) return;
                
                const content = block.attributes.content.replace(
                    /<mark class="search-replace-highlight"[^>]*>(.*?)<\/mark>/g,
                    '$1'
                );
                dispatch('core/block-editor').updateBlockAttributes(block.clientId, {
                    content: content
                });
            });
        } catch (error) {
            console.error('Error cleaning up highlights:', error);
        }
    };

    const findMatches = () => {
        try {
            cleanupHighlights();
            const blocks = select('core/block-editor').getBlocks();
            const newMatches = [];
            let totalOffset = 0;

            blocks.forEach(block => {
                if (!block.attributes?.content) return;
                
                const content = block.attributes.content;
                let regex;
                
                if (useRegex) {
                    try {
                        const flags = caseSensitive ? 'g' : 'gi';
                        regex = new RegExp(searchText, flags);
                    } catch (e) {
                        dispatch('core/notices').createErrorNotice(
                            __('Invalid regular expression', 'post-search-replace'),
                            { type: 'snackbar' }
                        );
                        return;
                    }
                } else {
                    const flags = caseSensitive ? 'g' : 'gi';
                    const escapedTerm = searchText.replace(/[.*+?^${}()|[\]\\]/g, '\\$&');
                    regex = new RegExp(escapedTerm, flags);
                }

                let match;
                while ((match = regex.exec(content)) !== null) {
                    newMatches.push({
                        index: totalOffset + match.index,
                        length: match[0].length,
                        text: match[0],
                        blockId: block.clientId,
                        blockOffset: match.index
                    });
                }
                
                totalOffset += content.length;
            });
            
            setMatches(newMatches);
            setCurrentMatch(0);

            if (newMatches.length > 0) {
                highlightCurrentMatch(newMatches[0]);
                scrollToMatch(0);
                dispatch('core/notices').createSuccessNotice(
                    sprintf(__('Found %d matches', 'post-search-replace'), newMatches.length),
                    { type: 'snackbar' }
                );
            } else {
                dispatch('core/notices').createNotice(
                    'info',
                    __('No matches found', 'post-search-replace'),
                    { type: 'snackbar' }
                );
            }
        } catch (error) {
            console.error('Error finding matches:', error);
            dispatch('core/notices').createErrorNotice(
                __('Error searching content', 'post-search-replace'),
                { type: 'snackbar' }
            );
        }
    };

    const replaceOne = () => {
        if (!matches.length || currentMatch >= matches.length) return;
        
        try {
            const match = matches[currentMatch];
            const block = select('core/block-editor').getBlock(match.blockId);
            
            if (!block || !block.attributes?.content) return;
            
            const content = block.attributes.content;
            const before = content.slice(0, match.blockOffset);
            const after = content.slice(match.blockOffset + match.length);
            const newContent = before + replaceText + after;
            
            dispatch('core/block-editor').updateBlockAttributes(match.blockId, {
                content: newContent
            });
            
            // Refresh matches
            findMatches();
            
            dispatch('core/notices').createSuccessNotice(
                __('Replacement complete', 'post-search-replace'),
                { type: 'snackbar' }
            );
        } catch (error) {
            console.error('Error replacing match:', error);
            dispatch('core/notices').createErrorNotice(
                __('Error performing replacement', 'post-search-replace'),
                { type: 'snackbar' }
            );
        }
    };

    const replaceAll = () => {
        try {
            const blocks = select('core/block-editor').getBlocks();
            
            blocks.forEach(block => {
                if (!block.attributes?.content) return;
                
                const content = block.attributes.content;
                let newContent = content;
                
                if (useRegex) {
                    try {
                        const flags = caseSensitive ? 'g' : 'gi';
                        const regex = new RegExp(searchText, flags);
                        newContent = content.replace(regex, replaceText);
                    } catch (e) {
                        dispatch('core/notices').createErrorNotice(
                            __('Invalid regular expression', 'post-search-replace'),
                            { type: 'snackbar' }
                        );
                        return;
                    }
                } else {
                    const flags = caseSensitive ? 'g' : 'gi';
                    const escapedTerm = searchText.replace(/[.*+?^${}()|[\]\\]/g, '\\$&');
                    const regex = new RegExp(escapedTerm, flags);
                    newContent = content.replace(regex, replaceText);
                }
                
                if (content !== newContent) {
                    dispatch('core/block-editor').updateBlockAttributes(block.clientId, {
                        content: newContent
                    });
                }
            });
            
            cleanupHighlights();
            setMatches([]);
            setCurrentMatch(0);
            
            dispatch('core/notices').createSuccessNotice(
                __('All matches replaced', 'post-search-replace'),
                { type: 'snackbar' }
            );
        } catch (error) {
            console.error('Error replacing all matches:', error);
            dispatch('core/notices').createErrorNotice(
                __('Error performing replacement', 'post-search-replace'),
                { type: 'snackbar' }
            );
        }
    };

    const navigateToMatch = (direction) => {
        try {
            let newIndex;
            if (direction === 'next') {
                newIndex = Math.min(currentMatch + 1, matches.length - 1);
            } else {
                newIndex = Math.max(currentMatch - 1, 0);
            }

            if (newIndex !== currentMatch) {
                cleanupHighlights();
                setCurrentMatch(newIndex);
                highlightCurrentMatch(matches[newIndex]);
                scrollToMatch(newIndex);
            }
        } catch (error) {
            console.error('Error navigating matches:', error);
        }
    };

    const scrollToMatch = (matchIndex) => {
        try {
            const match = matches[matchIndex];
            if (!match) return;

            dispatch('core/block-editor').selectBlock(match.blockId);
            const blockElement = document.querySelector(`[data-block="${match.blockId}"]`);
            if (blockElement) {
                blockElement.scrollIntoView({ behavior: 'smooth', block: 'center' });
            }
        } catch (error) {
            console.error('Error scrolling to match:', error);
        }
    };

    const highlightCurrentMatch = (match) => {
        try {
            if (!match) return;
            
            const block = select('core/block-editor').getBlock(match.blockId);
            if (!block || !block.attributes?.content) return;
            
            const content = block.attributes.content;
            const newContent = content.slice(0, match.blockOffset) +
                '<mark class="search-replace-highlight">' +
                content.slice(match.blockOffset, match.blockOffset + match.length) +
                '</mark>' +
                content.slice(match.blockOffset + match.length);

            dispatch('core/block-editor').updateBlockAttributes(match.blockId, {
                content: newContent
            });
        } catch (error) {
            console.error('Error highlighting match:', error);
        }
    };

    const escapeRegExp = (string) => {
        return string.replace(/[.*+?^${}()|[\]\\]/g, '\\$&');
    };

    return wp.element.createElement(wp.element.Fragment, {},
        wp.element.createElement(wp.editPost.PluginSidebarMoreMenuItem, {
            target: 'post-search-replace',
            icon: 'search'
        }, __('Search & Replace', 'post-search-replace')),
        
        wp.element.createElement(wp.editPost.PluginSidebar, {
            name: 'post-search-replace',
            title: __('Search & Replace', 'post-search-replace'),
            icon: 'search'
        },
            wp.element.createElement(wp.components.PanelBody, {},
                wp.element.createElement(wp.components.TextControl, {
                    label: __('Search for:', 'post-search-replace'),
                    value: searchText,
                    onChange: setSearchText
                }),
                
                wp.element.createElement(wp.components.TextControl, {
                    label: __('Replace with:', 'post-search-replace'),
                    value: replaceText,
                    onChange: setReplaceText
                }),
                
                wp.element.createElement(wp.components.ToggleControl, {
                    label: __('Use regular expressions', 'post-search-replace'),
                    checked: useRegex,
                    onChange: setUseRegex
                }),
                
                wp.element.createElement(wp.components.ToggleControl, {
                    label: __('Case sensitive', 'post-search-replace'),
                    checked: caseSensitive,
                    onChange: setCaseSensitive
                }),
                
                wp.element.createElement('div', { className: 'search-replace-buttons' },
                    wp.element.createElement(wp.components.Button, {
                        isPrimary: true,
                        onClick: findMatches,
                        disabled: !searchText
                    }, __('Find All', 'post-search-replace')),
                    
                    wp.element.createElement('div', { className: 'navigation-buttons' },
                        wp.element.createElement(wp.components.Button, {
                            isSecondary: true,
                            onClick: () => navigateToMatch('prev'),
                            disabled: currentMatch === 0 || matches.length === 0
                        }, __('Previous', 'post-search-replace')),
                        wp.element.createElement(wp.components.Button, {
                            isSecondary: true,
                            onClick: () => navigateToMatch('next'),
                            disabled: currentMatch === matches.length - 1 || matches.length === 0
                        }, __('Next', 'post-search-replace'))
                    ),
                    
                    wp.element.createElement(wp.components.Button, {
                        isSecondary: true,
                        onClick: replaceOne,
                        disabled: !matches.length
                    }, __('Replace', 'post-search-replace')),
                    
                    wp.element.createElement(wp.components.Button, {
                        isSecondary: true,
                        onClick: replaceAll,
                        disabled: !matches.length
                    }, __('Replace All', 'post-search-replace'))
                ),
                
                matches.length > 0 && wp.element.createElement('div', 
                    { className: 'search-replace-status' },
                    sprintf(__('Match %d of %d', 'post-search-replace'), 
                        currentMatch + 1, 
                        matches.length
                    )
                )
            )
        )
    );
}

wp.plugins.registerPlugin('post-search-replace', {
    render: SearchReplacePlugin,
    icon: 'search'
});