jQuery(document).ready(function($) {
    // Add button to editor toolbar
    var button = $('<button/>', {
        text: 'Search & Replace',
        class: 'button search-replace-button',
        click: function(e) {
            e.preventDefault();
            openSearchReplaceDialog();
        }
    });

    $('.wp-editor-tools').append(button);
    
    function openSearchReplaceDialog() {
        var matches = [];
        var currentMatchIndex = 0;
        var dialog;

        function cleanup() {
            var content = getContent();
            content = content.replace(/<mark class="search-highlight"[^>]*>(.*?)<\/mark>/g, '$1');
            setContent(content);
        }

        function getContent() {
            return tinyMCE && tinyMCE.activeEditor && !tinyMCE.activeEditor.isHidden() 
                ? tinyMCE.activeEditor.getContent()
                : $('#content').val();
        }

        function setContent(content) {
            if (tinyMCE && tinyMCE.activeEditor && !tinyMCE.activeEditor.isHidden()) {
                tinyMCE.activeEditor.setContent(content);
            } else {
                $('#content').val(content);
            }
        }

        dialog = $('<div/>', {
            title: postSearchReplace.dialogTitle,
            class: 'search-replace-dialog'
        }).appendTo('body');

        var content = $('<div/>', {
            class: 'search-replace-content'
        }).appendTo(dialog);

        // Search input
        content.append('<label>Search for:</label>');
        var searchInput = $('<input/>', {
            type: 'text',
            class: 'widefat search-input'
        }).appendTo(content);

        // Replace input
        content.append('<label>Replace with:</label>');
        var replaceInput = $('<input/>', {
            type: 'text',
            class: 'widefat replace-input'
        }).appendTo(content);

        // Options
        var optionsDiv = $('<div/>', { class: 'search-options' }).appendTo(content);
        
        var regexOption = $('<label><input type="checkbox"> Use regular expressions</label>')
            .appendTo(optionsDiv);
        
        var caseOption = $('<label><input type="checkbox"> Case sensitive</label>')
            .appendTo(optionsDiv);

        // Status display
        var statusDiv = $('<div/>', { 
            class: 'status-display',
            text: postSearchReplace.noMatches
        }).appendTo(content);

        // Navigation buttons
        var navDiv = $('<div/>', { class: 'nav-buttons' }).appendTo(content);
        
        var prevButton = $('<button/>', {
            text: 'Previous',
            class: 'button',
            disabled: true,
            click: function() {
                if (currentMatchIndex > 0) {
                    currentMatchIndex--;
                    highlightMatch(currentMatchIndex);
                    updateNavigationButtons();
                    scrollToMatch(currentMatchIndex);
                }
            }
        }).appendTo(navDiv);

        var nextButton = $('<button/>', {
            text: 'Next',
            class: 'button',
            disabled: true,
            click: function() {
                if (currentMatchIndex < matches.length - 1) {
                    currentMatchIndex++;
                    highlightMatch(currentMatchIndex);
                    updateNavigationButtons();
                    scrollToMatch(currentMatchIndex);
                }
            }
        }).appendTo(navDiv);

        function updateNavigationButtons() {
            prevButton.prop('disabled', currentMatchIndex === 0 || matches.length === 0);
            nextButton.prop('disabled', currentMatchIndex === matches.length - 1 || matches.length === 0);
        }

        function scrollToMatch(index) {
            var match = matches[index];
            if (!match) return;

            var isVisual = tinyMCE && tinyMCE.activeEditor && !tinyMCE.activeEditor.isHidden();
            
            if (isVisual) {
                var editor = tinyMCE.activeEditor;
                editor.selection.collapse(true);
                
                // Find the highlighted element
                var highlightedElement = editor.dom.select('mark.search-highlight')[0];
                if (highlightedElement) {
                    editor.selection.scrollIntoView(highlightedElement, {
                        align: 'center',
                        margin: 30
                    });
                    
                    // Select the highlighted text
                    editor.selection.select(highlightedElement);
                }
            } else {
                var textarea = $('#content')[0];
                textarea.setSelectionRange(match.index, match.index + match.length);
                
                // Improved scrolling calculation
                var lines = textarea.value.substr(0, match.index).split('\n');
                var lineNumber = lines.length;
                var lineHeight = parseInt(window.getComputedStyle(textarea).lineHeight);
                var scrollPosition = (lineNumber - 1) * lineHeight;
                
                textarea.scrollTop = scrollPosition - (textarea.clientHeight / 2);
                textarea.focus();
            }
        }

        function findMatches() {
            cleanup();
            var content = getContent();
            var searchTerm = searchInput.val();
            var useRegex = regexOption.find('input').is(':checked');
            var caseSensitive = caseOption.find('input').is(':checked');

            try {
                let regex;
                if (useRegex) {
                    const flags = caseSensitive ? 'g' : 'gi';
                    regex = new RegExp(searchTerm, flags);
                } else {
                    const flags = caseSensitive ? 'g' : 'gi';
                    const escapedTerm = searchTerm.replace(/[.*+?^${}()|[\]\\]/g, '\\$&');
                    regex = new RegExp(escapedTerm, flags);
                }

                matches = [];
                let match;
                while ((match = regex.exec(content)) !== null) {
                    matches.push({
                        index: match.index,
                        text: match[0],
                        length: match[0].length
                    });
                }

                statusDiv.text(matches.length > 0 ? 
                    `Found ${matches.length} matches` : 
                    postSearchReplace.noMatches
                );
                
                currentMatchIndex = 0;
                updateNavigationButtons();
                
                if (matches.length > 0) {
                    highlightMatch(0);
                    scrollToMatch(0);
                }

                return matches.length > 0;
            } catch (error) {
                statusDiv.text(postSearchReplace.invalidRegex);
                matches = [];
                currentMatchIndex = 0;
                updateNavigationButtons();
                return false;
            }
        }

        function highlightMatch(index) {
            cleanup();
            if (!matches[index]) return;

            var content = getContent();
            var match = matches[index];
            
            var before = content.slice(0, match.index);
            var matchText = content.slice(match.index, match.index + match.length);
            var after = content.slice(match.index + match.length);
            
            var newContent = before + 
                           '<mark class="search-highlight">' + 
                           matchText + 
                           '</mark>' + 
                           after;

            setContent(newContent);
            statusDiv.text(`Match ${index + 1} of ${matches.length}`);
        }

        function replaceCurrent() {
            if (!matches[currentMatchIndex]) return;

            try {
                var content = getContent();
                var match = matches[currentMatchIndex];
                var replaceWith = replaceInput.val();
                
                var before = content.slice(0, match.index);
                var after = content.slice(match.index + match.length);
                var newContent = before + replaceWith + after;

                setContent(newContent);
                
                // Preserve cursor position for next match
                var nextMatchIndex = currentMatchIndex < matches.length - 1 ? currentMatchIndex : 0;
                
                // Refresh matches and update UI
                findMatches();
                
                if (matches.length > 0) {
                    currentMatchIndex = Math.min(nextMatchIndex, matches.length - 1);
                    highlightMatch(currentMatchIndex);
                    updateNavigationButtons();
                    scrollToMatch(currentMatchIndex);
                }
                
                statusDiv.text(postSearchReplace.replacementDone);
            } catch (error) {
                statusDiv.text('Error performing replacement');
            }
        }

        function replaceAll() {
            try {
                var content = getContent();
                var searchTerm = searchInput.val();
                var replaceWith = replaceInput.val();
                var useRegex = regexOption.find('input').is(':checked');
                var caseSensitive = caseOption.find('input').is(':checked');

                let regex;
                if (useRegex) {
                    const flags = caseSensitive ? 'g' : 'gi';
                    regex = new RegExp(searchTerm, flags);
                } else {
                    const flags = caseSensitive ? 'g' : 'gi';
                    const escapedTerm = searchTerm.replace(/[.*+?^${}()|[\]\\]/g, '\\$&');
                    regex = new RegExp(escapedTerm, flags);
                }

                var newContent = content.replace(regex, replaceWith);
                setContent(newContent);
                
                // Clear matches and update UI
                cleanup();
                matches = [];
                currentMatchIndex = 0;
                updateNavigationButtons();
                statusDiv.text(postSearchReplace.replacementDone);

            } catch (error) {
                statusDiv.text('Error performing replacement');
            }
        }

        // Initialize dialog
        dialog.dialog({
            width: 500,
            modal: true,
            buttons: [
                {
                    text: postSearchReplace.findButton,
                    click: findMatches
                },
                {
                    text: postSearchReplace.replaceButton,
                    click: replaceCurrent
                },
                {
                    text: postSearchReplace.replaceAllButton,
                    click: replaceAll
                },
                {
                    text: postSearchReplace.closeButton,
                    click: function() {
                        cleanup();
                        $(this).dialog('close');
                        dialog.remove();
                    }
                }
            ],
            close: function() {
                cleanup();
                dialog.remove();
            }
        });

        // Add event listener for search input
        searchInput.on('keyup', function(e) {
            if (e.keyCode === 13) { // Enter key
                findMatches();
            }
        });
    }
});