<?php
/**
 * Plugin Name: Post Search Replace
 * Plugin URI: 
 * Description: Adds search and replace functionality to the WordPress editor
 * Version: 1.0.1
 * Author: Robert Wallace
 * License: GPL v2 or later
 * Text Domain: post-search-replace
 */

if (!defined('ABSPATH')) {
    exit;
}

class WP_Post_Search_Replace {
    
    /**
     * Supported post types
     *
     * @var array
     */
    private $supported_post_types;

    public function __construct() {
        // Initialize supported post types
        $this->supported_post_types = apply_filters('post_search_replace_post_types', array(
            'post',
            'page'
        ));

        add_action('init', array($this, 'init'));
    }

    public function init() {
        // Ensure we register for block editor
        add_action('enqueue_block_editor_assets', array($this, 'enqueue_block_editor_assets'));
        add_action('admin_enqueue_scripts', array($this, 'enqueue_classic_editor_assets'));
    }

    /**
     * Check if current screen is a supported post type
     */
private function is_supported_post_type() {
    $screen = get_current_screen();
    return $screen && 
           in_array($screen->post_type, $this->supported_post_types) && 
           $screen->is_block_editor;  // Add this check
}

    /**
     * Get plugin file URL with cache busting
     */
    private function get_plugin_file_url($file) {
        $file_path = plugin_dir_path(__FILE__) . $file;
        $version = file_exists($file_path) ? filemtime($file_path) : '1.0.1';
        return plugins_url($file, __FILE__) . '?ver=' . $version;
    }

public function enqueue_block_editor_assets() {
    if (!$this->is_supported_post_type()) {
        return;
    }

    // Enqueue the block editor script with wp-i18n as a dependency
    wp_enqueue_script(
        'post-search-replace-block',
        $this->get_plugin_file_url('js/block-editor.js'),
        array(
            'wp-blocks',
            'wp-element',
            'wp-editor',
            'wp-components',
            'wp-i18n',
            'wp-plugins',
            'wp-edit-post',
            'wp-data'
        ),
        filemtime(plugin_dir_path(__FILE__) . 'js/block-editor.js'),  // Use file modification time for versioning
        true
    );

    // Set up translations
    if (function_exists('wp_set_script_translations')) {
        wp_set_script_translations(
            'post-search-replace-block',
            'post-search-replace',
            plugin_dir_path(__FILE__) . 'languages'
        );
    }

    // Enqueue styles
    wp_enqueue_style(
        'post-search-replace',
        $this->get_plugin_file_url('css/search-replace.css'),
        array(),
        filemtime(plugin_dir_path(__FILE__) . 'css/search-replace.css')
    );
}
    public function enqueue_classic_editor_assets($hook) {
        if (!in_array($hook, array('post.php', 'post-new.php')) || !$this->is_supported_post_type()) {
            return;
        }

        wp_enqueue_script('jquery-ui-dialog');
        wp_enqueue_style('wp-jquery-ui-dialog');

        wp_enqueue_script(
            'post-search-replace-classic',
            $this->get_plugin_file_url('js/classic-editor.js'),
            array('jquery', 'jquery-ui-dialog'),
            null,
            true
        );

        wp_enqueue_style(
            'post-search-replace',
            $this->get_plugin_file_url('css/search-replace.css'),
            array('wp-jquery-ui-dialog'),
            null
        );

        wp_localize_script(
            'post-search-replace-classic',
            'postSearchReplace',
            array(
                'dialogTitle' => __('Search & Replace', 'post-search-replace'),
                'findButton' => __('Find', 'post-search-replace'),
                'replaceButton' => __('Replace', 'post-search-replace'),
                'replaceAllButton' => __('Replace All', 'post-search-replace'),
                'closeButton' => __('Close', 'post-search-replace'),
                'noMatches' => __('No matches found', 'post-search-replace'),
                'invalidRegex' => __('Invalid regular expression', 'post-search-replace'),
                'replacementDone' => __('Replacement complete', 'post-search-replace'),
                'searchLabel' => __('Search for:', 'post-search-replace'),
                'replaceLabel' => __('Replace with:', 'post-search-replace'),
                'useRegexLabel' => __('Use regular expressions', 'post-search-replace'),
                'caseSensitiveLabel' => __('Case sensitive', 'post-search-replace'),
                'prevMatchLabel' => __('Previous match', 'post-search-replace'),
                'nextMatchLabel' => __('Next match', 'post-search-replace'),
                'currentMatch' => __('Match %1$s of %2$s', 'post-search-replace')
            )
        );
    }
}

// Initialize the plugin
new WP_Post_Search_Replace();